package parser;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import parser.Grammar.NonTerminal;

import com.google.common.collect.HashBasedTable;
import com.google.common.collect.Table;

public class GotoDFA {
	
	Grammar grammar;
	public Set<LR0Set> states = new HashSet<LR0Set>();
	LR0Set initialState;
	Table<LR0Set, Alphabet, LR0Set> transitions = HashBasedTable.create();
	
	public GotoDFA(Grammar grammar) {
		this.grammar = grammar;
		generateLR0StateSpace();
	}
	
	public void addState(LR0Set state){
		assert(!states.contains(state));
		states.add(state);
	}
	
	/**
	 * 
	 * @param source
	 * @param letter
	 * @return Returns the letter-successor of source, or null if no such mapping exists.
	 */
	public LR0Set getSuccessor(LR0Set source, Alphabet letter){
		return transitions.get(source, letter);
	}
	
	/**
	 * This method computes all LR0 sets and transitions between them.
	 */
	public void generateLR0StateSpace(){
		//TODO
	}
	
	private LR0Set epsilonClosure(LR0Set set){
		//Hint: might be worthwhile to implement a method like this
		return null;
	}
	
	/**
	 * From a set of rules of the form
	 * A -> ab | aC
	 * create the "fresh" items
	 * A -> * ab, A -> * aC
	 * 
	 * @param lhs
	 * @return A set of items with nothing left of the dot
	 */
	private LR0Set freshItems(NonTerminal lhs){
		LR0Set result = new LR0Set();
		List<Rule> rules = grammar.getRules(lhs);
		for(Rule rule : rules){
			result.add(LR0Item.freshItem(rule));
		}
		return result;
	}
	
	public int countConflicts(){
		int counter = 0;
		for(LR0Set lr0set : states){
			if(lr0set.hasConflicts())
				counter++;
		}
		return counter;
	}

	public void printLR0Sets(){
		for(LR0Set set : states){
			System.out.println(set.getName() + " " + set);
		}
	}
	
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("digraph{");
		for(LR0Set state : states){
			builder.append(state.getName());
			builder.append(" [label=\"");
			builder.append(state.toString());
			builder.append("\"];\n");
		}
		for(Table.Cell<LR0Set, Alphabet, LR0Set> transition: transitions.cellSet()){
			builder.append(transition.getRowKey().getName());
			builder.append(" -> ");
			builder.append(transition.getValue().getName());
			builder.append(" [label=\"");
			builder.append(transition.getColumnKey());
			builder.append("\"];\n");
		}
		builder.append("}");
		return builder.toString();
	}
}
